@@ -490,4 +490,35 @@ QDEVICE_HELP_INFO = """  QDevice participates in quorum decisions. With the assi
 
 
 SSH_OPTION = "-o StrictHostKeyChecking=no"
+
+
+CLOUD_AWS = "amazon-web-services"
+CLOUD_AZURE = "microsoft-azure"
+CLOUD_GCP = "google-cloud-platform"
+
+
+RED = '\033[31m'
+YELLOW = '\033[33m'
+GREEN = '\033[32m'
+END = '\033[0m'
+
+
+CIB_QUERY = "cibadmin -Q"
+CIB_UPGRADE = "crm configure upgrade force"
+CIB_RAW_FILE = "/var/lib/pacemaker/cib/cib.xml"
+XML_NODE_PATH = "/cib/configuration/nodes/node"
+XML_STATUS_PATH = "/cib/status/node_state"
+XML_NODE_QUERY_STANDBY_PATH = "//nodes/node[@id='{node_id}']/instance_attributes/nvpair[@name='standby']"
+XML_STATUS_QUERY_STANDBY_PATH = "//status/node_state[@id='{node_id}']/transient_attributes/instance_attributes/nvpair[@name='standby']"
+CRM_MON_ONE_SHOT = "crm_mon -1"
+CRM_MON_XML_OUTPUT= "crm_mon --output-as=xml"
+STONITH_TIMEOUT_DEFAULT = 60
+PCMK_DELAY_MAX = 30
+DLM_CONTROLD_RA = "ocf::pacemaker:controld"
+LVMLOCKD_RA = "ocf::heartbeat:lvmlockd"
+HA_USER = "hacluster"
+HA_GROUP = "haclient"
+SCHEMA_MIN_VER_SUPPORT_OCF_1_1 = "pacemaker-3.7"
+REJOIN_COUNT = 60
+REJOIN_INTERVAL = 10
 # vim:ts=4:sw=4:et:
