@@ -7,7 +7,6 @@ import re
 from argparse import ArgumentParser, RawDescriptionHelpFormatter
 from . import command
 from . import utils
-from .msg import err_buf
 from . import scripts
 from . import completers as compl
 from . import bootstrap
@@ -16,6 +15,10 @@ from .cibconfig import cib_factory
 from . import constants
 
 
+from . import log
+logger = log.setup_logger(__name__)
+
+
 class ArgParser(ArgumentParser):
     def format_epilog(self, formatter):
         return self.epilog or ""
@@ -26,13 +29,67 @@ def parse_options(parser, args):
         options, args = parser.parse_known_args(list(args))
     except:
         return None, None
-    if options.help:
+    if hasattr(options, 'help') and options.help:
         parser.print_help()
         return None, None
     utils.check_space_option_value(options)
     return options, args
 
 
+def parse_option_for_nodes(context, *args):
+    """
+    Parse option for nodes
+    Return a node list
+    """
+    action_type = context.get_command_name()
+    action_target = "node" if action_type in ["standby", "online"] else "cluster service"
+    action = "{} {}".format(action_type, action_target)
+    usage_template = """
+Specify node(s) on which to {action}.
+If no nodes are specified, {action} on the local node.
+If --all is specified, {action} on all nodes."""
+    addtion_usage = ""
+    if action_type == "standby":
+        usage_template += """
+\n\nAdditionally, you may specify a lifetime for the standby---if set to
+"reboot", the node will be back online once it reboots. "forever" will
+keep the node in standby after reboot. The life time defaults to
+"forever"."""
+        addtion_usage = " [lifetime]"
+
+    parser = ArgParser(description=usage_template.format(action=action),
+            usage="{} [--all | <node>... ]{}".format(action_type, addtion_usage),
+            add_help=False,
+            formatter_class=RawDescriptionHelpFormatter)
+    parser.add_argument("-h", "--help", action="store_true", dest="help", help="Show this help message")
+    parser.add_argument("--all", help="To {} on all nodes".format(action), action="store_true", dest="all")
+
+    options, args = parse_options(parser, args)
+    if options is None or args is None:
+        raise utils.TerminateSubCommand
+    if options.all and args:
+        context.fatal_error("Should either use --all or specific node(s)")
+
+    # return local node
+    if not options.all and not args:
+        return [utils.this_node()]
+    member_list = utils.list_cluster_nodes()
+    if not member_list:
+        context.fatal_error("Cannot get the node list from cluster")
+    for node in args:
+        if node not in member_list:
+            context.fatal_error("Node \"{}\" is not a cluster node".format(node))
+
+    node_list = member_list if options.all else args
+    for node in node_list:
+        try:
+            utils.ping_node(node)
+        except ValueError as err:
+            logger.warning(str(err))
+            node_list.remove(node)
+    return node_list
+
+
 def _remove_completer(args):
     try:
         n = utils.list_cluster_nodes()
@@ -88,78 +145,102 @@ class Cluster(command.UI):
         self._inventory_target = None
 
     @command.skill_level('administrator')
-    def do_start(self, context):
+    def do_start(self, context, *args):
         '''
-        Starts the cluster services on this node
+        Starts the cluster services on all nodes or specific node(s)
         '''
-        try:
-            if utils.service_is_active("pacemaker.service"):
-                err_buf.info("Cluster services already started")
-                return
-            utils.start_service("pacemaker")
-            if utils.is_qdevice_configured():
-                utils.start_service("corosync-qdevice")
-            err_buf.info("Cluster services started")
-        except IOError as err:
-            context.fatal_error(str(err))
+        service_check_list = ["pacemaker.service"]
+        start_qdevice = False
+        if utils.is_qdevice_configured():
+            start_qdevice = True
+            service_check_list.append("corosync-qdevice.service")
 
-        # TODO: optionally start services on all nodes or specific node
+        node_list = parse_option_for_nodes(context, *args)
+        for node in node_list[:]:
+            if all([utils.service_is_active(srv, remote_addr=node) for srv in service_check_list]):
+                logger.info("Cluster services already started on {}".format(node))
+                node_list.remove(node)
+        if not node_list:
+            return
+
+        if start_qdevice:
+            utils.start_service("corosync-qdevice", node_list=node_list)
+        bootstrap.start_pacemaker(node_list)
+        for node in node_list:
+            logger.info("Cluster services started on {}".format(node))
 
     @command.skill_level('administrator')
-    def do_stop(self, context):
+    def do_stop(self, context, *args):
         '''
-        Stops the cluster services on this node
+        Stops the cluster services on all nodes or specific node(s)
         '''
-        try:
-            if not utils.service_is_active("corosync.service"):
-                err_buf.info("Cluster services already stopped")
-                return
-            if utils.service_is_active("corosync-qdevice"):
-                utils.stop_service("corosync-qdevice")
-            utils.stop_service("corosync")
-            err_buf.info("Cluster services stopped")
-        except IOError as err:
-            context.fatal_error(str(err))
+        node_list = parse_option_for_nodes(context, *args)
+        for node in node_list[:]:
+            if not utils.service_is_active("corosync.service", remote_addr=node):
+                if utils.service_is_active("sbd.service", remote_addr=node):
+                    utils.stop_service("corosync", remote_addr=node)
+                    logger.info("Cluster services stopped on {}".format(node))
+                else:
+                    logger.info("Cluster services already stopped on {}".format(node))
+                node_list.remove(node)
+        if not node_list:
+            return
 
-        # TODO: optionally stop services on all nodes or specific node
+        if not utils.get_dc(timeout=5):
+            logger.error("No DC found currently, please wait if the cluster is still starting")
+            return False
+
+        # When dlm running and quorum is lost, before stop cluster service, should set
+        # enable_quorum_fencing=0, enable_quorum_lockspace=0 for dlm config option
+        if utils.is_dlm_running() and not utils.is_quorate():
+            logger.debug("Quorum is lost; Set enable_quorum_fencing=0 and enable_quorum_lockspace=0 for dlm")
+            utils.set_dlm_option(enable_quorum_fencing=0, enable_quorum_lockspace=0)
+
+        # Stop pacemaker since it can make sure cluster has quorum until stop corosync
+        utils.stop_service("pacemaker", node_list=node_list)
+        # Then, stop qdevice if is active
+        if utils.service_is_active("corosync-qdevice.service"):
+            utils.stop_service("corosync-qdevice.service", node_list=node_list)
+        # Last, stop corosync
+        utils.stop_service("corosync", node_list=node_list)
+
+        for node in node_list:
+            logger.info("Cluster services stopped on {}".format(node))
 
     @command.skill_level('administrator')
-    def do_restart(self, context):
+    def do_restart(self, context, *args):
         '''
-        Restarts the cluster services on this node
+        Restarts the cluster services on all nodes or specific node(s)
         '''
-        self.do_stop(context)
-        self.do_start(context)
+        parse_option_for_nodes(context, *args)
+        self.do_stop(context, *args)
+        self.do_start(context, *args)
+
+    def _enable_disable_common(self, context, *args):
+        '''
+        Common part for enable and disable
+        '''
+        node_list = parse_option_for_nodes(context, *args)
+        action = context.get_command_name()
+        utils.cluster_run_cmd("systemctl {} pacemaker.service".format(action), node_list)
+        if utils.service_is_available("corosync-qdevice.service") and (utils.is_qdevice_configured() or action == "disable"):
+            utils.cluster_run_cmd("systemctl {} corosync-qdevice.service".format(action), node_list)
+        for node in node_list:
+            logger.info("Cluster services %s on %s", action+'d', node)
 
     @command.skill_level('administrator')
-    def do_enable(self, context):
+    def do_enable(self, context, *args):
         '''
         Enable the cluster services on this node
         '''
-        try:
-            utils.enable_service("pacemaker.service")
-            if utils.is_qdevice_configured():
-                utils.enable_service("corosync-qdevice.service")
-            err_buf.info("Cluster services enabled")
-        except IOError as err:
-            context.fatal_error(str(err))
-
-        # TODO: optionally enable services on all nodes or specific node
+        self._enable_disable_common(context, *args)
 
     @command.skill_level('administrator')
-    def do_disable(self, context):
+    def do_disable(self, context, *args):
         '''
         Disable the cluster services on this node
         '''
-        try:
-            utils.disable_service("pacemaker.service")
-            if utils.is_qdevice_configured():
-                utils.disable_service("corosync-qdevice.service")
-            err_buf.info("Cluster services disabled")
-        except IOError as err:
-            context.fatal_error(str(err))
-
-        # TODO: optionally disable services on all nodes or specific node
+        self._enable_disable_common(context, *args)
 
     def _args_implicit(self, context, args, name):
         '''
@@ -231,8 +312,9 @@ Note:
         network_group.add_argument("-i", "--interface", dest="nic_list", metavar="IF", action="append", choices=utils.interface_choice(),
                                    help="Bind to IP address on interface IF. Use -i second time for second interface")
         network_group.add_argument("-u", "--unicast", action="store_true", dest="unicast",
-                                   help="Configure corosync to communicate over unicast (UDP), and not multicast. " +
-                                   "Default is multicast unless an environment where multicast cannot be used is detected.")
+                                   help="Configure corosync to communicate over unicast(udpu). This is the default transport type")
+        network_group.add_argument("-U", "--multicast", action="store_true", dest="multicast",
+                                   help="Configure corosync to communicate over multicast. Default is unicast")
         network_group.add_argument("-A", "--admin-ip", dest="admin_ip", metavar="IP",
                                    help="Configure IP address as an administration virtual IP")
         network_group.add_argument("-M", "--multi-heartbeats", action="store_true", dest="second_heartbeat",
@@ -275,7 +357,7 @@ Note:
             stage = args[0]
         if stage == "vgfs":
             stage = "ocfs2"
-            err_buf.warning("vgfs stage was deprecated and is an alias of ocfs2 stage now")
+            logger.warning("vgfs stage was deprecated and is an alias of ocfs2 stage now")
         if stage not in bootstrap.INIT_STAGES and stage != "":
             parser.error("Invalid stage (%s)" % (stage))
 
@@ -305,7 +387,7 @@ Note:
             for node in nodelist:
                 if node == utils.this_node():
                     continue
-                bootstrap.status("\n\nAdd node {} (may prompt for root password):".format(node))
+                logger.info("\n\nAdd node {} (may prompt for root password):".format(node))
                 if not self._add_node(node, yes_to_all=options.yes_to_all):
                     return False
 
@@ -660,7 +742,7 @@ to get the geo cluster configuration.""",
     @command.skill_level('expert')
     def do_run(self, context, cmd, *nodes):
         '''
-        Execute the given command on all nodes/specific node, report outcome
+        Execute the given command on all nodes/specific node(s), report outcome
         '''
         try:
             import parallax
@@ -687,15 +769,15 @@ to get the geo cluster configuration.""",
                 break
         for host, result in parallax.call(hosts, cmd, opts).items():
             if isinstance(result, parallax.Error):
-                err_buf.error("[%s]: %s" % (host, result))
+                logger.error("[%s]: %s" % (host, result))
             else:
                 if result[0] != 0:
-                    err_buf.error("[%s]: rc=%s\n%s\n%s" % (host, result[0], utils.to_ascii(result[1]), utils.to_ascii(result[2])))
+                    logger.error("[%s]: rc=%s\n%s\n%s" % (host, result[0], utils.to_ascii(result[1]), utils.to_ascii(result[2])))
                 else:
                     if not result[1]:
-                        err_buf.ok("[%s]" % host)
+                        logger.info("[%s]" % host)
                     else:
-                        err_buf.ok("[%s]\n%s" % (host, utils.to_ascii(result[1])))
+                        logger.info("[%s]\n%s" % (host, utils.to_ascii(result[1])))
 
     def do_copy(self, context, local_file, *nodes):
         '''
