@@ -10,10 +10,12 @@ from . import utils
 from . import xmlutil
 from . import ui_utils
 from . import options
-
-from .msg import common_error, common_err, common_info, common_debug
-from .msg import no_prog_err
 from .cibconfig import cib_factory
+from . import log
+
+
+logger = log.setup_logger(__name__)
+logger_utils = log.LoggerUtils(logger)
 
 
 def rm_meta_attribute(node, attr, l, force_children=False):
@@ -66,8 +68,7 @@ def set_deep_meta_attr_node(target_node, attr, value):
                         (config.core.manage_children == "ask" and
                         utils.ask("'%s' conflicts with '%s' attribute. Remove '%s' for child resource %s?" %
                                 (attr, conflicting_attr, conflicting_attr, c.get("id")))):
-                    common_debug("force remove meta attr %s from %s" %
-                                (conflicting_attr, c.get("id")))
+                    logger.debug("force remove meta attr %s from %s", conflicting_attr, c.get("id"))
                     rm_meta_attribute(c, conflicting_attr, nvpair_l, force_children=True)
         # remove the same attributes from all children (if wanted)
         odd_children = get_children_with_attr(target_node, attr)
@@ -76,8 +77,7 @@ def set_deep_meta_attr_node(target_node, attr, value):
                     (config.core.manage_children == "ask" and
                      utils.ask("Do you want to override '%s' for child resource %s?" %
                                (attr, c.get("id")))):
-                common_debug("force remove meta attr %s from %s" %
-                             (attr, c.get("id")))
+                logger.debug("force remove meta attr %s from %s", attr, c.get("id"))
                 rm_meta_attribute(c, attr, nvpair_l, force_children=True)
     xmlutil.rmnodes(list(set(nvpair_l)))
     if '' != conflicting_attr:
@@ -147,19 +147,19 @@ def set_deep_meta_attr(rsc, attr, value, commit=True):
 
     objs = cib_factory.find_objects(rsc)
     if objs is None:
-        common_error("CIB is not valid!")
+        logger.error("CIB is not valid!")
         return False
     while any(obj for obj in objs if obj.obj_type == 'tag'):
         objs = list(flatten(resolve(obj) for obj in objs))
     objs = list(filter(is_resource, objs))
-    common_debug("set_deep_meta_attr: %s" % (', '.join([obj.obj_id for obj in objs])))
+    logger.debug("set_deep_meta_attr: %s", ', '.join([obj.obj_id for obj in objs]))
     if not objs:
-        common_error("Resource not found: %s" % (rsc))
+        logger.error("Resource not found: %s", rsc)
         return False
 
     ok = all(update_obj(obj) for obj in objs)
     if not ok:
-        common_error("Failed to update meta attributes for %s" % (rsc))
+        logger.error("Failed to update meta attributes for %s", rsc)
         return False
 
     if not commit:
@@ -167,7 +167,7 @@ def set_deep_meta_attr(rsc, attr, value, commit=True):
 
     ok = cib_factory.commit()
     if not ok:
-        common_error("Failed to commit updates to %s" % (rsc))
+        logger.error("Failed to commit updates to %s", rsc)
         return False
     return True
 
@@ -251,7 +251,7 @@ class RscMgmt(command.UI):
     def requires(self):
         for program in ('crm_resource', 'crm_attribute'):
             if not utils.is_program(program):
-                no_prog_err(program)
+                logger_utils.no_prog_err(program)
                 return False
         return True
 
@@ -297,7 +297,7 @@ class RscMgmt(command.UI):
         if commit and rc:
             ok = cib_factory.commit()
             if not ok:
-                common_error("Failed to commit updates to %s" % (rsc))
+                logger.error("Failed to commit updates to %s", rsc)
             return ok
         return rc
 
@@ -321,12 +321,12 @@ class RscMgmt(command.UI):
     @command.completers(compl.resources)
     def do_restart(self, context, *resources):
         "usage: restart <rsc> [<rsc> ...]"
-        common_info("ordering %s to stop" % ", ".join(resources))
+        logger.info("ordering %s to stop", ", ".join(resources))
         if not self._commit_meta_attrs(context, resources, "target-role", "Stopped"):
             return False
         if not utils.wait4dc("stop", not options.batch):
             return False
-        common_info("ordering %s to start" % ", ".join(resources))
+        logger.info("ordering %s to start", ", ".join(resources))
         return self._commit_meta_attrs(context, resources, "target-role", "Started")
 
     @command.wait
@@ -335,10 +335,11 @@ class RscMgmt(command.UI):
         "usage: promote <rsc>"
         if not utils.is_name_sane(rsc):
             return False
-        if not xmlutil.RscState().is_ms(rsc):
-            common_err("%s is not a master-slave resource" % rsc)
+        if not xmlutil.RscState().is_ms_or_promotable_clone(rsc):
+            logger.error("%s is not a promotable resource", rsc)
             return False
-        return utils.ext_cmd(self.rsc_setrole % (rsc, "Master")) == 0
+        role = "Promoted" if config.core.OCF_1_1_SUPPORT else "Master"
+        return utils.ext_cmd(self.rsc_setrole % (rsc, role)) == 0
 
     def do_scores(self, context):
         "usage: scores"
@@ -363,10 +364,11 @@ class RscMgmt(command.UI):
         "usage: demote <rsc>"
         if not utils.is_name_sane(rsc):
             return False
-        if not xmlutil.RscState().is_ms(rsc):
-            common_err("%s is not a master-slave resource" % rsc)
+        if not xmlutil.RscState().is_ms_or_promotable_clone(rsc):
+            logger.error("%s is not a promotable resource", rsc)
             return False
-        return utils.ext_cmd(self.rsc_setrole % (rsc, "Slave")) == 0
+        role = "Unpromoted" if config.core.OCF_1_1_SUPPORT else "Slave"
+        return utils.ext_cmd(self.rsc_setrole % (rsc, role)) == 0
 
     @command.completers(compl.resources)
     def do_manage(self, context, rsc):
@@ -378,27 +380,38 @@ class RscMgmt(command.UI):
         "usage: unmanage <rsc>"
         return self._commit_meta_attr(context, rsc, "is-managed", "false")
 
-    @command.alias('migrate')
-    @command.skill_level('administrator')
-    @command.wait
-    @command.completers_repeating(compl.resources, compl.nodes,
-                                  compl.choice(['reboot', 'forever', 'force']))
-    def do_move(self, context, rsc, *args):
-        """usage: move <rsc> [<node>] [<lifetime>] [force]"""
+    def move_or_ban(self, context, rsc, *args):
+        """
+        Common codes for move or ban action
+        """
         if not utils.is_name_sane(rsc):
             return False
+        cmd_map_dict = {'move': self.rsc_migrate,
+                'ban': self.rsc_ban}
+        action = context.get_command_name()
+        action_cap = action.capitalize()
+        action_cmd = cmd_map_dict[action]
+        usage = "usage: {} <rsc> [<node>] [<lifetime>] [force]".format(action)
         node = None
+        lifetime = None
+
         argl = list(args)
         force = "force" in utils.fetch_opts(argl, ["force"]) or config.core.force
-        lifetime = utils.fetch_lifetime_opt(argl)
-        if len(argl) > 0:
+        if len(argl) >= 3:
+            context.fatal_error(usage)
+        if len(argl) == 2:  # must be <node> <lifetime>
             node = argl[0]
             if not xmlutil.is_our_node(node):
                 context.fatal_error("Not our node: " + node)
+            lifetime = utils.fetch_lifetime_opt(argl)
+        elif len(argl) == 1: # could be <node> or <lifetime>
+            if xmlutil.is_our_node(argl[0]):
+                node = argl[0]
+            else:
+                lifetime = utils.fetch_lifetime_opt(argl)
 
-        if context.get_command_name() == 'move':
-            if not node and not force:
-                context.fatal_error("No target node: Move requires either a target node or 'force'")
+        if action == "move" and not node and not force:
+            context.fatal_error("No target node: {} requires either a target node or 'force'".format(action_cap))
 
         opts = ''
         if node:
@@ -407,43 +420,30 @@ class RscMgmt(command.UI):
             opts = "%s --lifetime '%s'" % (opts, lifetime)
         if force or config.core.force:
             opts = "%s --force" % opts
-        rc = utils.ext_cmd(self.rsc_migrate % (rsc, opts))
+        rc = utils.ext_cmd(action_cmd % (rsc, opts))
         if rc == 0:
             if node:
-                common_info("Move constraint created for %s to %s" % (rsc, node))
+                logger.info("%s constraint created for %s to %s", action_cap, rsc, node)
             else:
-                common_info("Move constraint created for %s" % (rsc))
+                logger.info("%s constraint created for %s", action_cap, rsc)
+            logger.info("Use `crm resource clear %s` to remove this constraint", rsc)
         return rc == 0
 
+    @command.alias('migrate')
+    @command.skill_level('administrator')
+    @command.wait
+    @command.completers_repeating(compl.resources, compl.nodes)
+    def do_move(self, context, rsc, *args):
+        """usage: move <rsc> [<node>] [<lifetime>] [force]"""
+        return self.move_or_ban(context, rsc, *args)
+
+
     @command.skill_level('administrator')
     @command.wait
     @command.completers_repeating(compl.resources, compl.nodes)
     def do_ban(self, context, rsc, *args):
         """usage: ban <rsc> [<node>] [<lifetime>] [force]"""
-        if not utils.is_name_sane(rsc):
-            return False
-        node = None
-        argl = list(args)
-        force = "force" in utils.fetch_opts(argl, ["force"]) or config.core.force
-        lifetime = utils.fetch_lifetime_opt(argl)
-        if len(argl) > 0:
-            node = argl[0]
-            if not xmlutil.is_our_node(node):
-                context.fatal_error("Not our node: " + node)
-        opts = ''
-        if node:
-            opts = "--node '%s'" % node
-        if lifetime:
-            opts = "%s --lifetime '%s'" % (opts, lifetime)
-        if force:
-            opts = "%s --force" % opts
-        rc = utils.ext_cmd(self.rsc_ban % (rsc, opts))
-        if rc == 0:
-            if node:
-                common_info("Ban constraint created for %s on %s" % (rsc, node))
-            else:
-                common_info("Ban constraint created for %s" % (rsc))
-        return rc == 0
+        return self.move_or_ban(context, rsc, *args)
 
     @command.alias('unmove', 'unban', 'unmigrate')
     @command.skill_level('administrator')
@@ -455,7 +455,7 @@ class RscMgmt(command.UI):
             return False
         rc = utils.ext_cmd(self.rsc_unmigrate % rsc)
         if rc == 0:
-            common_info("Removed migration constraints for %s" % (rsc))
+            logger.info("Removed migration constraints for %s", rsc)
         return rc == 0
 
     @command.skill_level('administrator')
@@ -628,10 +628,10 @@ class RscMgmt(command.UI):
             return None
         rsc = cib_factory.find_object(rsc_id)
         if not rsc:
-            common_err("resource %s does not exist" % rsc_id)
+            logger.error("resource %s does not exist", rsc_id)
             return None
         if rsc.obj_type != "primitive":
-            common_err("element %s is not a primitive resource" % rsc_id)
+            logger.error("element %s is not a primitive resource", rsc_id)
             return None
         return rsc
 
@@ -644,7 +644,8 @@ class RscMgmt(command.UI):
         return rsc.add_operation(n)
 
     def _trace_resource(self, context, rsc_id, rsc):
-        op_nodes = rsc.node.xpath('.//op')
+        """Enable RA tracing for a specified resource."""
+        op_nodes = rsc.node.xpath('operations/op')
 
         def trace(name):
             for o in op_nodes:
@@ -654,14 +655,15 @@ class RscMgmt(command.UI):
                 context.fatal_error("Failed to add trace for %s:%s" % (rsc_id, name))
         trace('start')
         trace('stop')
-        if xmlutil.is_ms(rsc.node):
+        if xmlutil.is_ms_or_promotable_clone(rsc.node):
             trace('promote')
             trace('demote')
         for op_node in op_nodes:
             rsc.set_op_attr(op_node, constants.trace_ra_attr, "1")
 
     def _trace_op(self, context, rsc_id, rsc, op):
-        op_nodes = rsc.node.xpath('.//op[@name="%s"]' % (op))
+        """Enable RA tracing for a specified operation."""
+        op_nodes = rsc.node.xpath('operations/op[@name="%s"]' % (op))
         if not op_nodes:
             if op == 'monitor':
                 context.fatal_error("No monitor operation configured for %s" % (rsc_id))
@@ -671,6 +673,7 @@ class RscMgmt(command.UI):
             rsc.set_op_attr(op_node, constants.trace_ra_attr, "1")
 
     def _trace_op_interval(self, context, rsc_id, rsc, op, interval):
+        """Enable RA tracing for an operation with the exact interval."""
         op_node = xmlutil.find_operation(rsc.node, op, interval)
         if op_node is None and utils.crm_msec(interval) != 0:
             context.fatal_error("Operation %s with interval %s not found in %s" % (op, interval, rsc_id))
@@ -698,25 +701,45 @@ class RscMgmt(command.UI):
             self._trace_op_interval(context, rsc_id, rsc, op, interval)
         if not cib_factory.commit():
             return False
-        if op is not None:
-            common_info("Trace for %s:%s is written to %s/trace_ra/" %
-                        (rsc_id, op, config.path.heartbeat_dir))
-        else:
-            common_info("Trace for %s is written to %s/trace_ra/" %
-                        (rsc_id, config.path.heartbeat_dir))
+        rsc_type = rsc.node.get("type")
+        logger.info("Trace for %s%s is written to %s/trace_ra/%s", rsc_id, ":"+op if op else "", config.path.heartbeat_dir, rsc_type)
         if op is not None and op != "monitor":
-            common_info("Trace set, restart %s to trace the %s operation" % (rsc_id, op))
+            logger.info("Trace set, restart %s to trace the %s operation", rsc_id, op)
         else:
-            common_info("Trace set, restart %s to trace non-monitor operations" % (rsc_id))
+            logger.info("Trace set, restart %s to trace non-monitor operations", rsc_id)
         return True
 
     def _remove_trace(self, rsc, op_node):
-        from lxml import etree
-        common_debug("op_node: %s" % (xmlutil.xml_tostring(op_node)))
+        logger.debug("op_node: %s", xmlutil.xml_tostring(op_node))
         op_node = rsc.del_op_attr(op_node, constants.trace_ra_attr)
         if rsc.is_dummy_operation(op_node):
             rsc.del_operation(op_node)
 
+    def _untrace_resource(self, context, rsc_id, rsc):
+        """Disable RA tracing for a specified resource."""
+        op_nodes = rsc.node.xpath(
+            'operations/op[instance_attributes/nvpair[@name="%s"]]' %
+            (constants.trace_ra_attr))
+        for op_node in op_nodes:
+            self._remove_trace(rsc, op_node)
+
+    def _untrace_op(self, context, rsc_id, rsc, op):
+        """Disable RA tracing for a specified operation."""
+        op_nodes = rsc.node.xpath('operations/op[@name="%s"]' % (op))
+        if not op_nodes:
+            context.fatal_error("Operation %s not found in %s" % (op, rsc_id))
+        for op_node in op_nodes:
+            self._remove_trace(rsc, op_node)
+
+    def _untrace_op_interval(self, context, rsc_id, rsc, op, interval):
+        """Disable RA tracing for an operation with the exact interval."""
+        op_node = xmlutil.find_operation(rsc.node, op, interval)
+        if op_node is None:
+            context.fatal_error(
+                "Operation %s with interval %s not found in %s" %
+                (op, interval, rsc_id))
+        self._remove_trace(rsc, op_node)
+
     @command.completers(compl.primitives, _raoperations)
     def do_untrace(self, context, rsc_id, op=None, interval=None):
         'usage: untrace <rsc> [<op>] [<interval>]'
@@ -725,19 +748,15 @@ class RscMgmt(command.UI):
             return False
         if op == "probe":
             op = "monitor"
+            if interval is None:
+                interval = "0"
         if op is None:
-            n = 0
-            for tn in rsc.node.xpath('.//*[@%s]' % (constants.trace_ra_attr)):
-                self._remove_trace(rsc, tn)
-                n += 1
-            for tn in rsc.node.xpath('.//*[@name="%s"]' % (constants.trace_ra_attr)):
-                if tn.getparent().getparent().tag == 'op':
-                    self._remove_trace(rsc, tn.getparent().getparent())
-                    n += 1
+            self._untrace_resource(context, rsc_id, rsc)
+        elif interval is None:
+            self._untrace_op(context, rsc_id, rsc, op)
         else:
-            op_node = xmlutil.find_operation(rsc.node, op, interval=interval)
-            if op_node is None:
-                common_err("operation %s does not exist in %s" % (op, rsc.obj_id))
-                return False
-            self._remove_trace(rsc, op_node)
-        return cib_factory.commit()
+            self._untrace_op_interval(context, rsc_id, rsc, op, interval)
+        if not cib_factory.commit():
+            return False
+        logger.info("Stop tracing %s%s", rsc_id, " for operation "+op if op else "")
+        return True
