@@ -21,14 +21,20 @@ import string
 from pathlib import Path
 from contextlib import contextmanager, closing
 from stat import S_ISBLK
+from lxml import etree
 from . import config
 from . import userdir
 from . import constants
 from . import options
 from . import term
 from . import parallax
-from .msg import common_warn, common_info, common_debug, common_err, err_buf
+from distutils.version import LooseVersion
 from .constants import SSH_OPTION
+from . import log
+
+
+logger = log.setup_logger(__name__)
+logger_utils = log.LoggerUtils(logger)
 
 
 class TerminateSubCommand(Exception):
@@ -212,7 +218,7 @@ def ask(msg):
     If not interactive and core.force is false, always return false.
     """
     if config.core.force:
-        common_info("%s [YES]" % (msg))
+        logger.info("%s [YES]", msg)
         return True
     if not can_ask():
         return False
@@ -254,8 +260,8 @@ def multi_input(prompt=''):
             text = input(prompt)
         except EOFError:
             return None
-        err_buf.incr_lineno()
         if options.regression_tests:
+            logger_utils.incr_lineno()
             print(".INP:", text)
             sys.stdout.flush()
             sys.stderr.flush()
@@ -386,7 +392,7 @@ def ensure_sudo_readable(f):
         try:
             os.chown(f, uid, -1)
         except os.error as err:
-            common_err('Failed setting temporary file permissions: %s' % (err))
+            logger.error('Failed setting temporary file permissions: %s', err)
             return False
     return True
 
@@ -394,7 +400,7 @@ def ensure_sudo_readable(f):
 def pipe_string(cmd, s):
     rc = -1  # command failed
     cmd = add_sudo(cmd)
-    common_debug("piping string to %s" % cmd)
+    logger.debug("piping string to %s", cmd)
     if options.regression_tests:
         print(".EXT", cmd)
     p = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE)
@@ -407,7 +413,7 @@ def pipe_string(cmd, s):
         rc = p.returncode
     except IOError as msg:
         if "Broken pipe" not in str(msg):
-            common_err(msg)
+            logger.error(msg)
     return rc
 
 
@@ -419,7 +425,7 @@ def filter_string(cmd, s, stderr_on=True, shell=True):
     else:
         stderr = subprocess.PIPE
     cmd = add_sudo(cmd)
-    common_debug("pipe through %s" % cmd)
+    logger.debug("pipe through %s", cmd)
     if options.regression_tests:
         print(".EXT", cmd)
     p = subprocess.Popen(cmd,
@@ -440,11 +446,10 @@ def filter_string(cmd, s, stderr_on=True, shell=True):
         rc = p.returncode
     except OSError as err:
         if err.errno != os.errno.EPIPE:
-            common_err(err.strerror)
-        common_info("from: %s" % cmd)
+            logger.error(err.strerror)
+        logger.error("from: %s", cmd)
     except Exception as msg:
-        common_err(msg)
-        common_info("from: %s" % cmd)
+        logger.error("from: %s: %s", cmd, str(msg))
     return rc, to_ascii(outp)
 
 
@@ -458,7 +463,7 @@ def str2tmp(_str, suffix=".pcmk"):
     try:
         f = os.fdopen(fd, "w")
     except IOError as msg:
-        common_err(msg)
+        logger.error(msg)
         return
     f.write(s)
     if not s.endswith('\n'):
@@ -537,7 +542,7 @@ def str2file(s, fname, mod=0o644):
             dst.write(to_ascii(s))
         os.chmod(fname, mod)
     except IOError as msg:
-        common_err(msg)
+        logger.error(msg)
         return False
     return True
 
@@ -550,7 +555,7 @@ def file2str(fname, noerr=True):
         f = open(fname, "r")
     except IOError as msg:
         if not noerr:
-            common_err(msg)
+            logger.error(msg)
         return None
     s = f.readline()
     f.close()
@@ -564,7 +569,7 @@ def file2list(fname):
     try:
         return open(fname).read().split('\n')
     except IOError as msg:
-        common_err(msg)
+        logger.error(msg)
         return None
 
 
@@ -578,7 +583,7 @@ def safe_open_w(fname):
         try:
             f = open(fname, "w")
         except IOError as msg:
-            common_err(msg)
+            logger.error(msg)
             return None
     return f
 
@@ -590,21 +595,21 @@ def safe_close_w(f):
 
 def is_path_sane(name):
     if re.search(r"['`#*?$\[\];]", name):
-        common_err("%s: bad path" % name)
+        logger.error("%s: bad path", name)
         return False
     return True
 
 
 def is_filename_sane(name):
     if re.search(r"['`/#*?$\[\];]", name):
-        common_err("%s: bad filename" % name)
+        logger.error("%s: bad filename", name)
         return False
     return True
 
 
 def is_name_sane(name):
     if re.search("[']", name):
-        common_err("%s: bad name" % name)
+        logger.error("%s: bad name", name)
         return False
     return True
 
@@ -617,14 +622,14 @@ def show_dot_graph(dotfile, keep_file=False, desc="transition graph"):
         print(".EXT", cmd)
     subprocess.Popen(cmd, shell=True, bufsize=0,
                      stdin=None, stdout=None, stderr=None, close_fds=True)
-    common_info("starting %s to show %s" % (config.core.dotty, desc))
+    logger.info("starting %s to show %s", config.core.dotty, desc)
 
 
 def ext_cmd(cmd, shell=True):
     cmd = add_sudo(cmd)
     if options.regression_tests:
         print(".EXT", cmd)
-    common_debug("invoke: %s" % cmd)
+    logger.debug("invoke: %s", cmd)
     return subprocess.call(cmd, shell=shell)
 
 
@@ -681,17 +686,17 @@ def check_locker(lockdir):
     s = file2str(os.path.join(lockdir, _LOCKDIR, _PIDF))
     pid = convert2ints(s)
     if not isinstance(pid, int):
-        common_warn("history: removing malformed lock")
+        logger.warning("history: removing malformed lock")
         rmdir_r(os.path.join(lockdir, _LOCKDIR))
         return
     try:
         os.kill(pid, 0)
     except OSError as err:
         if err.errno == os.errno.ESRCH:
-            common_info("history: removing stale lock")
+            logger.info("history: removing stale lock")
             rmdir_r(os.path.join(lockdir, _LOCKDIR))
         else:
-            common_err("%s: %s" % (_LOCKDIR, err.strerror))
+            logger.error("%s: %s", _LOCKDIR, err.strerror)
 
 
 @contextmanager
@@ -710,7 +715,7 @@ def lock(lockdir):
                 return True
             except OSError as err:
                 if err.errno != os.errno.EEXIST:
-                    common_err("Failed to acquire lock to %s: %s" % (lockdir, err.strerror))
+                    logger.error("Failed to acquire lock to %s: %s", lockdir, err.strerror)
                     return False
                 time.sleep(0.1)
                 continue
@@ -794,11 +799,11 @@ def get_stdout(cmd, input_s=None, stderr_on=True, shell=True, raw=False):
     return proc.returncode, to_ascii(stdout_data).strip()
 
 
-def get_stdout_stderr(cmd, input_s=None, shell=True, raw=False):
+def get_stdout_stderr(cmd, input_s=None, shell=True, raw=False, no_reg=False):
     '''
     Run a cmd, return (rc, stdout, stderr)
     '''
-    if options.regression_tests:
+    if options.regression_tests and not no_reg:
         print(".EXT", cmd)
     proc = subprocess.Popen(cmd,
                             shell=shell,
@@ -828,13 +833,15 @@ def append_file(dest, src):
         open(dest, "a").write(open(src).read())
         return True
     except IOError as msg:
-        common_err("append %s to %s: %s" % (src, dest, msg))
+        logger.error("append %s to %s: %s", src, dest, msg)
         return False
 
 
-def get_dc():
+def get_dc(timeout=None):
     cmd = "crmadmin -D"
-    rc, s = get_stdout(add_sudo(cmd))
+    if timeout:
+        cmd += " -t {}".format(timeout)
+    rc, s, _ = get_stdout_stderr(add_sudo(cmd))
     if rc != 0:
         return None
     if not s.startswith("Designated"):
@@ -867,14 +874,14 @@ def wait4dc(what="", show_progress=True):
     '''
     dc = get_dc()
     if not dc:
-        common_warn("can't find DC")
+        logger.warning("can't find DC")
         return False
     cmd = "crm_attribute -Gq -t crm_config -n crmd-transition-delay 2> /dev/null"
     delay = get_stdout(add_sudo(cmd))[1]
     if delay:
         delaymsec = crm_msec(delay)
         if delaymsec > 0:
-            common_info("The crmd-transition-delay is configured. Waiting %d msec before check DC status." % delaymsec)
+            logger.info("The crmd-transition-delay is configured. Waiting %d msec before check DC status.", delaymsec)
             time.sleep(delaymsec // 1000)
     cnt = 0
     output_started = 0
@@ -884,18 +891,17 @@ def wait4dc(what="", show_progress=True):
     while True:
         dc = get_dc()
         if not dc:
-            common_warn("DC lost during wait")
+            logger.warning("DC lost during wait")
             return False
         cmd = "crmadmin -S %s" % dc
         rc, s = get_stdout(add_sudo(cmd))
         if not s.startswith("Status"):
-            common_warn("%s unexpected output: %s (exit code: %d)" %
-                        (cmd, s, rc))
+            logger.warning("%s unexpected output: %s (exit code: %d)", cmd, s, rc)
             return False
         try:
             dc_status = s.split()[-2]
         except:
-            common_warn("%s unexpected output: %s" % (cmd, s))
+            logger.warning("%s unexpected output: %s", cmd, s)
             return False
         if dc_status == "S_IDLE":
             if output_started:
@@ -940,22 +946,22 @@ def run_ptest(graph_s, nograph, scores, utilization, actions, verbosity):
         ptest = "%s | %s" % (ptest, actions_filter)
     if options.regression_tests:
         ptest = ">/dev/null %s" % ptest
-    common_debug("invoke: %s" % ptest)
+    logger.debug("invoke: %s", ptest)
     rc, s = get_stdout(ptest, input_s=graph_s)
     if rc != 0:
-        common_debug("'%s' exited with (rc=%d)" % (ptest, rc))
+        logger.debug("'%s' exited with (rc=%d)", ptest, rc)
         if actions and rc == 1:
-            common_warn("No actions found.")
+            logger.warning("No actions found.")
         else:
-            common_warn("Simulation was unsuccessful (RC=%d)." % (rc))
+            logger.warning("Simulation was unsuccessful (RC=%d).", rc)
     if dotfile:
         if os.path.getsize(dotfile) > 0:
             show_dot_graph(dotfile)
         else:
-            common_warn("ptest produced empty dot file")
+            logger.warning("ptest produced empty dot file")
     else:
         if not nograph:
-            common_info("install graphviz to see a transition graph")
+            logger.info("install graphviz to see a transition graph")
     if s:
         page_string(s)
     return True
@@ -1252,13 +1258,6 @@ def multicolumn(l):
             print(s)
 
 
-def find_value(pl, name):
-    for n, v in pl:
-        if n == name:
-            return v
-    return None
-
-
 def cli_replace_attr(pl, name, new_val):
     for i, attr in enumerate(pl):
         if attr[0] == name:
@@ -1329,7 +1328,7 @@ def datetime_to_timestamp(dt):
     try:
         return total_seconds(make_datetime_naive(dt) - datetime.datetime(1970, 1, 1))
     except Exception as e:
-        common_err("datetime_to_timestamp error: %s" % (e))
+        logger.error("datetime_to_timestamp error: %s", e)
         return None
 
 
@@ -1342,7 +1341,7 @@ def timestamp_to_datetime(ts):
     return make_datetime_naive(datetime.datetime.fromtimestamp(ts).replace(tzinfo=dateutil.tz.tzlocal()))
 
 
-def parse_time(t):
+def parse_time(t, quiet=False):
     '''
     Try to make sense of the user provided time spec.
     Use dateutil if available, otherwise strptime.
@@ -1366,19 +1365,20 @@ def parse_time(t):
             # convert to UTC from local time
             dt = dt - tz.tzlocal().utcoffset(dt)
     except ValueError as msg:
-        common_err("parse_time %s: %s" % (t, msg))
+        if not quiet:
+            logger.error("parse_time %s: %s", t, msg)
         return None
     except ImportError as msg:
         try:
             tm = time.strptime(t)
             dt = datetime.datetime(*tm[0:7])
         except ValueError as msg:
-            common_err("no dateutil, please provide times as printed by date(1)")
+            logger.error("no dateutil, please provide times as printed by date(1)")
             return None
     return dt
 
 
-def parse_to_timestamp(t):
+def parse_to_timestamp(t, quiet=False):
     '''
     Read a string and convert it into a UNIX timestamp.
     Added as an optimization of parse_time to avoid
@@ -1394,7 +1394,8 @@ def parse_to_timestamp(t):
         # convert to UTC from local time
         return total_seconds(dt - tz.tzlocal().utcoffset(dt) - datetime.datetime(1970, 1, 1))
     except ValueError as msg:
-        common_err("parse_time %s: %s" % (t, msg))
+        if not quiet:
+            logger.error("parse_time %s: %s", t, msg)
         return None
     except ImportError as msg:
         try:
@@ -1402,7 +1403,7 @@ def parse_to_timestamp(t):
             dt = datetime.datetime(*tm[0:7])
             return datetime_to_timestamp(dt)
         except ValueError as msg:
-            common_err("no dateutil, please provide times as printed by date(1)")
+            logger.error("no dateutil, please provide times as printed by date(1)")
             return None
 
 
@@ -1411,12 +1412,12 @@ def save_graphviz_file(ini_f, attr_d):
     Save graphviz settings to an ini file, if it does not exist.
     '''
     if os.path.isfile(ini_f):
-        common_err("%s exists, please remove it first" % ini_f)
+        logger.error("%s exists, please remove it first", ini_f)
         return False
     try:
         f = open(ini_f, "wb")
     except IOError as msg:
-        common_err(msg)
+        logger.error(msg)
         return False
     import configparser
     p = configparser.ConfigParser()
@@ -1427,10 +1428,10 @@ def save_graphviz_file(ini_f, attr_d):
     try:
         p.write(f)
     except IOError as msg:
-        common_err(msg)
+        logger.error(msg)
         return False
     f.close()
-    common_info("graphviz attributes saved to %s" % ini_f)
+    logger.info("graphviz attributes saved to %s", ini_f)
     return True
 
 
@@ -1445,7 +1446,7 @@ def load_graphviz_file(ini_f):
     try:
         p.read(ini_f)
     except Exception as msg:
-        common_err(msg)
+        logger.error(msg)
         return False, None
     _graph_d = {}
     for section in p.sections():
@@ -1468,17 +1469,17 @@ def get_pcmk_version(dflt):
     try:
         rc, s, err = get_stdout_stderr("%s version" % (cmd))
         if rc != 0:
-            common_err("%s exited with %d [err: %s][out: %s]" % (cmd, rc, err, s))
+            logger.error("%s exited with %d [err: %s][out: %s]", cmd, rc, err, s)
         else:
-            common_debug("pacemaker version: [err: %s][out: %s]" % (err, s))
+            logger.debug("pacemaker version: [err: %s][out: %s]", err, s)
             if err.startswith("CRM Version:"):
                 version = s.split()[0]
             else:
                 version = s.split()[2]
-            common_debug("found pacemaker version: %s" % version)
+            logger.debug("found pacemaker version: %s", version)
     except Exception as msg:
-        common_warn("could not get the pacemaker version, bad installation?")
-        common_warn(msg)
+        logger.warning("could not get the pacemaker version, bad installation?")
+        logger.warning(msg)
     return version
 
 
@@ -1495,7 +1496,7 @@ def get_cib_property(cib_f, attr, dflt):
     try:
         f = open(cib_f, "r")
     except IOError as msg:
-        common_err(msg)
+        logger.error(msg)
         return ver
     state = 0
     for s in f:
@@ -1525,7 +1526,7 @@ def get_cib_attributes(cib_f, tag, attr_l, dflt_l):
     try:
         f = open(cib_f, "rb").read()
     except IOError as msg:
-        common_err(msg)
+        logger.error(msg)
         return dflt_l
     if os.path.splitext(cib_f)[-1] == '.bz2':
         cib_bits = bz2.decompress(f)
@@ -1543,16 +1544,18 @@ def get_cib_attributes(cib_f, tag, attr_l, dflt_l):
     return val_l
 
 
+def is_larger_than_min_version(version, min_version):
+    return LooseVersion(version) >= LooseVersion(min_version)
+
+
 def is_min_pcmk_ver(min_ver, cib_f=None):
     if not constants.pcmk_version:
         if cib_f:
             constants.pcmk_version = get_cib_property(cib_f, "dc-version", "1.1.11")
-            common_debug("found pacemaker version: %s in cib: %s" %
-                         (constants.pcmk_version, cib_f))
+            logger.debug("found pacemaker version: %s in cib: %s", constants.pcmk_version, cib_f)
         else:
             constants.pcmk_version = get_pcmk_version("1.1.11")
-    from distutils.version import LooseVersion
-    return LooseVersion(constants.pcmk_version) >= LooseVersion(min_ver)
+    return is_larger_than_min_version(constants.pcmk_version, min_ver)
 
 
 def is_pcmk_118(cib_f=None):
@@ -1712,51 +1715,40 @@ def list_cluster_nodes():
     '''
     Returns a list of nodes in the cluster.
     '''
-    def getname(toks):
-        if toks and len(toks) >= 2:
-            return toks[1]
-        return None
-
-    try:
-        # when pacemaker running
-        rc, outp = stdout2list(['crm_node', '-l'], stderr_on=False, shell=False)
-        if rc == 0:
-            return [x for x in [getname(line.split()) for line in outp] if x and x != '(null)']
-
-        # when corosync running
-        ip_list = get_member_iplist()
-        if ip_list:
-            return ip_list
-
-        # static situation
-        cib_path = os.getenv('CIB_file', '/var/lib/pacemaker/cib/cib.xml')
+    from . import xmlutil
+    cib = None
+    rc, out, err = get_stdout_stderr(constants.CIB_QUERY)
+    # When cluster service running
+    if rc == 0:
+        cib = etree.fromstring(out)
+    # Static situation
+    else:
+        cib_path = os.getenv('CIB_file', constants.CIB_RAW_FILE)
         if not os.path.isfile(cib_path):
             return None
-        from . import xmlutil
-        node_list = []
         cib = xmlutil.file2cib_elem(cib_path)
-        if cib is None:
-            return None
-        for node in cib.xpath('/cib/configuration/nodes/node'):
-            name = node.get('uname') or node.get('id')
-            if node.get('type') == 'remote':
-                srv = cib.xpath("//primitive[@id='%s']/instance_attributes/nvpair[@name='server']" % (name))
-                if srv:
-                    continue
-            node_list.append(name)
-        return node_list
-    except OSError as msg:
-        raise ValueError("Error listing cluster nodes: %s" % (msg))
+    if cib is None:
+        return None
+
+    node_list = []
+    for node in cib.xpath(constants.XML_NODE_PATH):
+        name = node.get('uname') or node.get('id')
+        if node.get('type') == 'remote':
+            srv = cib.xpath("//primitive[@id='%s']/instance_attributes/nvpair[@name='server']" % (name))
+            if srv:
+                continue
+        node_list.append(name)
+    return node_list
 
 
-def cluster_run_cmd(cmd):
+def cluster_run_cmd(cmd, node_list=[]):
     """
     Run cmd in cluster nodes
     """
-    node_list = list_cluster_nodes()
-    if not node_list:
+    nodelist = node_list or list_cluster_nodes()
+    if not nodelist:
         raise ValueError("Failed to get node list from cluster")
-    parallax.parallax_call(node_list, cmd)
+    return parallax.parallax_call(nodelist, cmd)
 
 
 def list_cluster_nodes_except_me():
@@ -1803,7 +1795,7 @@ def running_on(resource):
             w = line[len(head):].split()
             if w:
                 nodes.append(w[0])
-    common_debug("%s running on: %s" % (resource, nodes))
+    logger.debug("%s running on: %s", resource, nodes)
     return nodes
 
 
@@ -1960,10 +1952,10 @@ def cluster_copy_file(local_path, nodes=None):
                                       local_path,
                                       local_path, opts).items():
         if isinstance(result, parallax.Error):
-            err_buf.error("Failed to push %s to %s: %s" % (local_path, host, result))
+            logger.error("Failed to push %s to %s: %s", local_path, host, result)
             ok = False
         else:
-            err_buf.ok(host)
+            logger.info(host)
     return ok
 
 
@@ -1995,11 +1987,6 @@ def gen_nodeid_from_ipv6(addr):
     return int(ipaddress.ip_address(addr)) % 1000000000
 
 
-# Set by detect_cloud()
-# to avoid multiple requests
-_ip_for_cloud = None
-
-
 def _cloud_metadata_request(uri, headers={}):
     try:
         import urllib2 as urllib
@@ -2018,6 +2005,51 @@ def _cloud_metadata_request(uri, headers={}):
         return None
 
 
+def detect_aws():
+    """
+    Detect if in AWS
+    """
+    system_version = get_stdout_or_raise_error("dmidecode -s system-version")
+    return re.search(r".*amazon.*", system_version) is not None
+
+
+def detect_azure():
+    """
+    Detect if in Azure
+    """
+    # Should check both system-manufacturer and chassis-asset-tag
+    # In some azure environment, dmidecode -s system-manufacturer
+    # might return American Megatrends Inc. instead of Microsoft Corporation in Azure.
+    # The better way is to check the result of dmidecode -s chassis-asset-tag is
+    # 7783-7084-3265-9085-8269-3286-77, aka. the ascii code of MSFT AZURE VM
+    system_manufacturer = get_stdout_or_raise_error("dmidecode -s system-manufacturer")
+    chassis_asset_tag = get_stdout_or_raise_error("dmidecode -s chassis-asset-tag")
+    if "microsoft corporation" in system_manufacturer.lower() or \
+            ''.join([chr(int(n)) for n in re.findall("\d\d", chassis_asset_tag)]) == "MSFT AZURE VM":
+        # To detect azure we also need to make an API request
+        result = _cloud_metadata_request(
+            "http://169.254.169.254/metadata/instance/network/interface/0/ipv4/ipAddress/0/privateIpAddress?api-version=2017-08-01&format=text",
+            headers={"Metadata": "true"})
+        if result:
+            return True
+    return False
+
+
+def detect_gcp():
+    """
+    Detect if in GCP
+    """
+    bios_vendor = get_stdout_or_raise_error("dmidecode -s bios-vendor")
+    if "Google" in bios_vendor:
+        # To detect GCP we also need to make an API request
+        result = _cloud_metadata_request(
+            "http://metadata.google.internal/computeMetadata/v1/instance/network-interfaces/0/ip",
+            headers={"Metadata-Flavor": "Google"})
+        if result:
+            return True
+    return False
+
+
 @memoize
 def detect_cloud():
     """
@@ -2037,33 +2069,17 @@ def detect_cloud():
     * google-cloud-platform
 
     """
-    global _ip_for_cloud
-
     if not is_program("dmidecode"):
         return None
-    rc, system_version = get_stdout("dmidecode -s system-version")
-    if re.search(r".*amazon.*", system_version) is not None:
-        return "amazon-web-services"
-    if rc != 0:
-        return None
-    rc, system_manufacturer = get_stdout("dmidecode -s system-manufacturer")
-    if rc == 0 and "microsoft corporation" in system_manufacturer.lower():
-        # To detect azure we also need to make an API request
-        result = _cloud_metadata_request(
-            "http://169.254.169.254/metadata/instance/network/interface/0/ipv4/ipAddress/0/privateIpAddress?api-version=2017-08-01&format=text",
-            headers={"Metadata": "true"})
-        if result:
-            _ip_for_cloud = result
-            return "microsoft-azure"
-    rc, bios_vendor = get_stdout("dmidecode -s bios-vendor")
-    if rc == 0 and "Google" in bios_vendor:
-        # To detect GCP we also need to make an API request
-        result = _cloud_metadata_request(
-            "http://metadata.google.internal/computeMetadata/v1/instance/network-interfaces/0/ip",
-            headers={"Metadata-Flavor": "Google"})
-        if result:
-            _ip_for_cloud = result
-            return "google-cloud-platform"
+    aws = detect_aws()
+    if aws:
+        return constants.CLOUD_AWS
+    azure = detect_azure()
+    if azure:
+        return constants.CLOUD_AZURE
+    gcp = detect_gcp()
+    if gcp:
+        return constants.CLOUD_GCP
     return None
 
 
@@ -2074,7 +2090,7 @@ def debug_timestamp():
 def get_member_iplist():
     rc, out, err= get_stdout_stderr("corosync-cmapctl -b runtime.totem.pg.mrp.srp.members")
     if rc != 0:
-        common_debug(err)
+        logger.debug(err)
         return None
 
     ip_list = []
@@ -2431,7 +2447,7 @@ class InterfacesInfo(object):
         else:
             if not self.nic_list:
                 self.get_interfaces_info()
-            common_warn("No default route configured. Using the first found nic")
+            logger.warning("No default route configured. Using the first found nic")
             self._default_nic_list = [self.nic_list[0]]
         return self._default_nic_list
 
@@ -2501,12 +2517,14 @@ class ServiceManager(object):
             "is_available": "list-unit-files"
             }
 
-    def __init__(self, service_name, remote_addr=None):
+    def __init__(self, service_name, remote_addr=None, node_list=[]):
         """
         Init function
+        When node_list set, execute action between nodes in parallel
         """
         self.service_name = service_name
         self.remote_addr = remote_addr
+        self.node_list = node_list
 
     def _do_action(self, action_type):
         """
@@ -2516,7 +2534,10 @@ class ServiceManager(object):
             raise ValueError("status_type should be {}".format('/'.join(list(self.ACTION_MAP.values()))))
 
         cmd = "systemctl {} {}".format(action_type, self.service_name)
-        if self.remote_addr:
+        if self.node_list:
+            cluster_run_cmd(cmd, self.node_list)
+            return True, None
+        elif self.remote_addr and self.remote_addr != this_node():
             prompt_msg = "Run \"{}\" on {}".format(cmd, self.remote_addr)
             rc, output, err = run_cmd_on_remote(cmd, self.remote_addr, prompt_msg)
         else:
@@ -2574,40 +2595,40 @@ class ServiceManager(object):
         return inst.is_active
 
     @classmethod
-    def start_service(cls, name, enable=False, remote_addr=None):
+    def start_service(cls, name, enable=False, remote_addr=None, node_list=[]):
         """
         Start service
         """
-        inst = cls(name, remote_addr)
+        inst = cls(name, remote_addr, node_list)
         if enable:
             inst.enable()
         inst.start()
 
     @classmethod
-    def stop_service(cls, name, disable=False, remote_addr=None):
+    def stop_service(cls, name, disable=False, remote_addr=None, node_list=[]):
         """
         Stop service
         """
-        inst = cls(name, remote_addr)
+        inst = cls(name, remote_addr, node_list)
         if disable:
             inst.disable()
         inst.stop()
 
     @classmethod
-    def enable_service(cls, name, remote_addr=None):
+    def enable_service(cls, name, remote_addr=None, node_list=[]):
         """
         Enable service
         """
-        inst = cls(name, remote_addr)
+        inst = cls(name, remote_addr, node_list)
         if inst.is_available and not inst.is_enabled:
             inst.enable()
 
     @classmethod
-    def disable_service(cls, name, remote_addr=None):
+    def disable_service(cls, name, remote_addr=None, node_list=[]):
         """
         Disable service
         """
-        inst = cls(name, remote_addr)
+        inst = cls(name, remote_addr, node_list)
         if inst.is_available and inst.is_enabled:
             inst.disable()
 
@@ -2645,21 +2666,21 @@ def ping_node(node):
         raise ValueError("host \"{}\" is unreachable: {}".format(node, err))
 
 
-def is_quorate(expected_votes, actual_votes):
+def calculate_quorate_status(expected_votes, actual_votes):
     """
     Given expected votes and actual votes, calculate if is quorated
     """
     return int(actual_votes)/int(expected_votes) > 0.5
 
 
-def get_stdout_or_raise_error(cmd, remote=None, success_val=0):
+def get_stdout_or_raise_error(cmd, remote=None, success_val_list=[0], no_raise=False):
     """
     Common function to get stdout from cmd or raise exception
     """
     if remote:
         cmd = "ssh {} root@{} \"{}\"".format(SSH_OPTION, remote, cmd)
-    rc, out, err = get_stdout_stderr(cmd)
-    if rc != success_val:
+    rc, out, err = get_stdout_stderr(cmd, no_reg=True)
+    if rc not in success_val_list and not no_raise:
         raise ValueError("Failed to run \"{}\": {}".format(cmd, err))
     return out
 
@@ -2668,29 +2689,10 @@ def get_quorum_votes_dict(remote=None):
     """
     Return a dictionary which contain expect votes and total votes
     """
-    out = get_stdout_or_raise_error("corosync-quorumtool -s", remote=remote)
+    out = get_stdout_or_raise_error("corosync-quorumtool -s", remote=remote, success_val_list=[0, 2])
     return dict(re.findall("(Expected|Total) votes:\s+(\d+)", out))
 
 
-def has_resource_running(ra_type=None):
-    """
-    Check if any RA is running
-    """
-    out = get_stdout_or_raise_error("crm_mon -1")
-    if ra_type:
-        return re.search("{}.*Started".format(ra_type), out) is not None
-    else:
-        return re.search("No active resources", out) is None
-
-
-def has_resource_configured(ra_type, peer=None):
-    """
-    Check if the RA configured
-    """
-    out = get_stdout_or_raise_error("crm_mon -1rR", remote=peer)
-    return re.search(ra_type, out) is not None
-
-
 def check_all_nodes_reachable():
     """
     Check if all cluster nodes are reachable
@@ -2722,8 +2724,11 @@ def has_stonith_running():
     """
     Check if any stonith device registered
     """
+    from . import sbd
     out = get_stdout_or_raise_error("stonith_admin -L")
-    return re.search("[1-9]+ fence device[s]* found", out) is not None
+    has_stonith_device = re.search("[1-9]+ fence device[s]* found", out) is not None
+    using_diskless_sbd = sbd.SBDManager.is_using_diskless_sbd()
+    return has_stonith_device or using_diskless_sbd
 
 
 def parse_append_action_argument(input_list, parse_re="[; ]"):
@@ -2845,7 +2850,15 @@ def is_dev_used_for_lvm(dev, peer=None):
     """
     Check if device is LV
     """
-    return get_dev_info(dev, "TYPE", peer=peer) == "lvm"
+    return "lvm" in get_dev_info(dev, "TYPE", peer=peer)
+
+
+def is_dev_a_plain_raw_disk_or_partition(dev, peer=None):
+    """
+    Check if device is a raw disk or partition
+    """
+    out = get_dev_info(dev, "TYPE", peer=peer)
+    return re.search("(disk|part)", out) is not None
 
 
 def compare_uuid_with_peer_dev(dev_list, peer):
@@ -2880,4 +2893,199 @@ def get_qdevice_sync_timeout():
     if not res:
         raise ValueError("Cannot find qdevice sync timeout")
     return int(int(res.group(1))/1000)
+
+
+def detect_virt():
+    """
+    Detect if running in virt environment
+    """
+    rc, _, _ = get_stdout_stderr("systemd-detect-virt")
+    return rc == 0
+
+
+def fatal(error_msg):
+    """
+    Raise exception to jump over this module,
+    handled by Context.run in ui_context.py
+    """
+    raise ValueError(error_msg)
+
+
+def is_standby(node):
+    """
+    Check if the node is already standby
+    """
+    out = get_stdout_or_raise_error("crm_mon -1")
+    return re.search(r'Node\s+{}:\s+standby'.format(node), out) is not None
+
+
+def get_dlm_option_dict():
+    """
+    Get dlm config option dictionary
+    """
+    out = get_stdout_or_raise_error("dlm_tool dump_config")
+    return dict(re.findall("(\w+)=(\w+)", out))
+
+
+def set_dlm_option(**kargs):
+    """
+    Set dlm option
+    """
+    dlm_option_dict = get_dlm_option_dict()
+    for option, value in kargs.items():
+        if option not in dlm_option_dict:
+            raise ValueError('"{}" is not dlm config option'.format(option))
+        if dlm_option_dict[option] != value:
+            get_stdout_or_raise_error('dlm_tool set_config "{}={}"'.format(option, value))
+
+
+def is_dlm_running():
+    """
+    Check if dlm ra controld is running
+    """
+    from . import xmlutil
+    return xmlutil.CrmMonXmlParser.is_resource_started(constants.DLM_CONTROLD_RA)
+
+
+def is_dlm_configured():
+    """
+    Check if dlm configured
+    """
+    from . import xmlutil
+    return xmlutil.CrmMonXmlParser.is_resource_configured(constants.DLM_CONTROLD_RA)
+
+
+def is_quorate():
+    """
+    Check if cluster is quorated
+    """
+    out = get_stdout_or_raise_error("corosync-quorumtool -s", success_val_list=[0, 2])
+    res = re.search(r'Quorate:\s+(.*)', out)
+    if res:
+        return res.group(1) == "Yes"
+    else:
+        raise ValueError("Failed to get quorate status from corosync-quorumtool")
+
+
+def is_2node_cluster_without_qdevice(removing=False):
+    """
+    Check if current cluster has two nodes without qdevice
+    """
+    current_num = len(list_cluster_nodes())
+    remove_num = 1 if removing else 0
+    qdevice_num = 1 if is_qdevice_configured() else 0
+    return (current_num - remove_num + qdevice_num) == 2
+
+
+def get_pcmk_delay_max(two_node_without_qdevice=False):
+    """
+    Get value of pcmk_delay_max
+    """
+    if service_is_active("pacemaker.service") and two_node_without_qdevice:
+        return constants.PCMK_DELAY_MAX
+    return 0
+
+
+def get_property(name):
+    """
+    Get cluster properties
+    """
+    cmd = "crm configure get_property " + name
+    rc, stdout, _ = get_stdout_stderr(cmd)
+    return stdout if rc == 0 else None
+
+
+def set_property(**kwargs):
+    """
+    Set cluster properties
+    """
+    set_str = ""
+    for key, value in kwargs.items():
+        set_str += "{}={} ".format(key, value)
+    cmd = "crm configure property " + set_str.strip().replace('_', '-')
+    get_stdout_or_raise_error(cmd)
+
+
+def check_no_quorum_policy_with_dlm():
+    """
+    Give warning when no-quorum-policy not freeze while configured DLM
+    """
+    if not is_dlm_configured():
+        return
+    res = get_property("no-quorum-policy")
+    if not res or res != "freeze":
+        logger.warning("The DLM cluster best practice suggests to set the cluster property \"no-quorum-policy=freeze\"")
+
+
+def set_property_conditionally(property_name, value_from_calculation):
+    """
+    Set cluster property if calculated value is larger then current cib value
+    """
+    _value = get_property(property_name)
+    value_from_cib = int(_value.strip('s')) if _value else 0
+    if value_from_cib < value_from_calculation:
+        cmd = "crm configure property {}={}".format(property_name, value_from_calculation)
+        get_stdout_or_raise_error(cmd)
+
+
+def get_systemd_timeout_start_in_sec(time_res):
+    """
+    Get the TimeoutStartUSec value in second unit
+    The origin format was like: 1min 30s
+    """
+    res_seconds = re.search("(\d+)s", time_res)
+    start_timeout = int(res_seconds.group(1)) if res_seconds else 0
+    res_min = re.search("(\d+)min", time_res)
+    start_timeout += 60 * int(res_min.group(1)) if res_min else 0
+    return start_timeout
+
+
+def is_ocf_1_1_cib_schema_detected():
+    """
+    Only turn on ocf_1_1 feature the cib schema version is pacemaker-3.7 or above
+    """
+    from .cibconfig import cib_factory
+    return is_larger_than_min_version(cib_factory.get_schema(), constants.SCHEMA_MIN_VER_SUPPORT_OCF_1_1)
+
+
+def handle_role_for_ocf_1_1(value, name='role'):
+    """
+    * Convert role from Promoted/Unpromoted to Master/Slave if schema doesn't support OCF 1.1
+    * Convert role from Master/Slave to Promoted/Unpromoted if ocf1.1 cib schema detected and OCF_1_1_SUPPORT is yes
+    """
+    role_names = ["role", "target-role"]
+    downgrade_dict = {"Promoted": "Master", "Unpromoted": "Slave"}
+    upgrade_dict = {v: k for k, v in downgrade_dict.items()}
+
+    if name not in role_names:
+        return value
+    if value in downgrade_dict and not is_ocf_1_1_cib_schema_detected():
+        logger.warning('Convert "%s" to "%s" since the current schema version is old and not upgraded yet. Please consider "%s"', value, downgrade_dict[value], constants.CIB_UPGRADE)
+        return downgrade_dict[value]
+    if value in upgrade_dict and is_ocf_1_1_cib_schema_detected() and config.core.OCF_1_1_SUPPORT:
+        logger.info('Convert deprecated "%s" to "%s"', value, upgrade_dict[value])
+        return upgrade_dict[value]
+
+    return value
+
+
+def diff_and_patch(orig_cib_str, current_cib_str):
+    """
+    Use crm_diff to generate patch, then apply
+    """
+    # In cibconfig.py, _patch_cib method doesn't include status section
+    # So here should make a function to handle common cases
+    cmd = "crm_diff -u -O '{}' -N '{}'".format(orig_cib_str, current_cib_str)
+    rc, cib_diff, err = get_stdout_stderr(cmd)
+    if rc == 0: # no difference
+        return True
+    if err:
+        logger.error("Failed to run crm_diff: %s", err)
+        return False
+    logger.debug("Diff: %s", cib_diff)
+    rc = pipe_string("cibadmin -p -P --force", cib_diff)
+    if rc != 0:
+        logger.error("Failed to patch")
+        return False
+    return True
 # vim:ts=4:sw=4:et:
